#include "state.hpp"

State::State () {
	//
}

void State::error () {
	ERROR = true;
	image.clear();
}

bool State::imageExists (string& image) {
    struct stat buffer;   
    return (stat (image.c_str(), &buffer) == 0); 
}

void State::setAction (json& msgData) {
	action = static_cast<Action>(msgData["action"]);
	switch (action) {
		case NONE:
			cout << "{ \"action\" : \"NONE\" }" << endl;
			break;
		case LOAD:
			cout << "{ \"action\" : \"LOAD\" }" << endl;
			break;
		case DISPLAY:
			cout << "{ \"action\" : \"DISPLAY\" }" << endl;
			break;
		case STOP:
			cout << "{ \"action\" : \"STOP\" }" << endl;
			break;
	}
}

void State::setImage (json& msgData) {
	image = msgData["image"];
	if (!imageExists(image)) {
		error();
		cerr << "{ \"error\" : \"Image " << msgData["image"] << " does not exist\" }" << endl;
		return;
	}
	cout << "{ \"image\" : \"" << image << "\" }" << endl;
}

void State::setMode (json& msgData) {
	mode = static_cast<Mode>(msgData["mode"]);
	cout << "{ \"mode\" : \"" << mode << "\" }"<< endl;
}

void State::setPosition (json& msgData) {
	x = msgData["position"]["x"];
	y = msgData["position"]["y"];
	w = msgData["position"]["w"];
	h = msgData["position"]["h"];
	cout << "{ \"position\" : { \"x\" : " << x << ", \"y\" : " << y << ", \"w\" : " << w << ", \"h\" : " << h << " } }" <<  endl;
}

void State::setExposure (json& msgData) {
	if (msgData["exposure"].size() == 0) {
		setNoExposure();
	} else if (msgData["exposure"].size() == 1) {
		exposure = { msgData["exposure"][0] };
		cout << "{ \"exposure\" : " << msgData["exposure"][0] << " }" << endl;
	} /*else if (msgData["exposure"].size() == 3) {
		exposure = { msgData["exposure"][0], msgData["exposure"][1], msgData["exposure"][2] };
		cout << "Exposure[0] = " << msgData["exposure"][0] << " ms, [1] = " << msgData["exposure"][1] << " ms, [2] = " << msgData["exposure"][2] << " ms" << endl;
	}*/ else {
		error();
		cerr << "{ \"error\" : \"Exposure array is incorrect length, " << msgData["exposure"].size() << "\" }" << endl;
		return;
	}
}

void State::setNoExposure () {
	exposure = {};
	cout << "{ \"exposure\" : null }" << endl;
}

void State::receiveMessage (string msgString) {
	ERROR = false;
	json msgData = json::parse(msgString);
	if (msgData.contains("action")) {
		setAction(msgData);
	} else {
		action = NONE;
		error();
		cerr << "{ \"error\" : Received invalid message: " << msgString << "\" }" << endl;
		return;
	}

	if (action == LOAD && msgData.contains("image")) {
		setImage(msgData);
	}

	if (action == LOAD && msgData.contains("mode")) {
		setMode(msgData);
	}

	if (action == LOAD && msgData.contains("position")) {
		setPosition(msgData);
	}

	if (action == DISPLAY && msgData.contains("exposure")) {
		setExposure(msgData);
	} else if (action == DISPLAY && !msgData.contains("exposure")) {
		setNoExposure();
	}

	setActive();
}

string State::createMessage (bool success) {
	json msgData = {
		{ "action", action },
		{ "success", success }
	};
	return msgData.dump();
}